#!/usr/bin/env python

#
# I simulate a SCHOTT LLS (LED Light Source).
#
# For usage, see my printUsage function or run me with my -h option.
#

import getopt
import asynchat
import asyncore
import os
import re
import signal
import socket
import sys

DEFAULT_PORT = 50441

def main(args):
  (port, isDebugOutputEnabled) = parseCommandLineArgs(args)
  d = ConnectionDispatcher(port, isDebugOutputEnabled)
  asyncore.loop()

def getProgramName(args=None):
  if args == None: args = sys.argv
  if len(args) == 0 or args[0] == "-c": return "PROGRAM_NAME"
  return os.path.basename(args[0])

def initializeSignalHandlers():
  signalToNameMap = {
    signal.SIGHUP:"SIGHUP",
    signal.SIGINT:"SIGINT",
    signal.SIGPIPE:"SIGPIPE",
    signal.SIGALRM:"SIGALRM",
    signal.SIGTERM:"SIGTERM",
    signal.SIGUSR1:"SIGUSR1",
    signal.SIGUSR2:"SIGUSR2"
  }

  def signalHandler(signum, frame):
    if signum in [signal.SIGINT, signal.SIGTERM]:
      print
      print "%s caught. Exiting." % signalToNameMap[signum]
      sys.exit(0)
    else:
      print >> sys.stderr
      print >> sys.stderr, "%s caught. Exiting." % signalToNameMap[signum]
      sys.exit(1)

  for each in signalToNameMap.keys():
    signal.signal(each, signalHandler)

def parseCommandLineArgs(args):
  (options, extra) = getopt.getopt(args[1:], "dp:h", ["debug", "port=",
    "help"])

  port = DEFAULT_PORT
  isDebugOutputEnabled = False

  for eachOptName, eachOptValue in options:
    if eachOptName in ("-d", "--debug"):
      isDebugOutputEnabled = True
    elif eachOptName in ("-p", "--port"):
      port = int(eachOptValue)
    elif eachOptName in ("-h", "--help"):
      printUsage(sys.stdout)
      sys.exit(0)

  if len(extra) > 0:
    print >> sys.stderr, "Error: unexpected command line argument \"%s\"" % \
      extra[0]
    printUsage(sys.stderr)
    sys.exit(1)

  return (port, isDebugOutputEnabled)

def printUsage(outStream):
  print >> outStream, """\
Usage: %s [-dph]

Options:
  -d,--debug        Print debug messages to stderr
  -p,--port=NUMBER  Listen on the specified port NUMBER for incoming
                    connections (default: %d)
  -h,--help         Print usage message and exit\
""" % (getProgramName(), DEFAULT_PORT)


class ConnectionDispatcher(asyncore.dispatcher):
  def __init__(self, port, debugOutputEnabled=False):
    asyncore.dispatcher.__init__(self)
    self.port = port
    self.debugOutputEnabled = debugOutputEnabled
    self.model = SchottLls()
    self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
    self.set_reuse_addr()
    self.bind(("", port))
    self.listen(5)

  def handle_accept(self):
    ConnectionSession(self.accept(), self.model, self.debugOutputEnabled)


class ConnectionSession(asynchat.async_chat):
  GetIntensityRe = re.compile(r"&i\?$", re.I)
  SetIntensityRe = re.compile(r"&i(?P<val>[0-9a-fA-F]{2})$", re.I)
  CommandRe = re.compile(r"&(?P<cmd>[a-zA-Z]+)", re.I)

  def __init__(self, (conn, addr), aSchottLls, debugOutputEnabled=False):
    asynchat.async_chat.__init__(self, conn)
    self.debugOutputEnabled = debugOutputEnabled
    self.set_terminator("\r")
    self.outputTerminator = "\r"
    self.model = aSchottLls
    self.buffer = ""

  def collect_incoming_data(self, data):
    self.buffer = self.buffer + data

  def found_terminator(self):
    data = self.buffer
    self.buffer = ""
    if self.debugOutputEnabled:
      print >> sys.stderr, "< \"%s\"" % \
        (data + self.get_terminator()).encode("string_escape")
    self.handleClientRequest(data)

  def handleClientRequest(self, request):
    request = request.strip()

    match = self.GetIntensityRe.match(request)
    if match != None:
      self.sendClientResponse("&i%02X" % self.model.getIntensity())
      return
    match = self.SetIntensityRe.match(request)
    if match != None:
      self.model.setIntensity(int(match.group("val"), 16))
      self.sendClientResponse("&i%02X" % self.model.getIntensity())
      return
    match = self.CommandRe.match(request)
    if match != None:
      self.sendClientResponse("&nc%s" % match.group("cmd"))
    print >> sys.stderr, "error: unknown command \"%s\"" % \
      request.encode("string_escape")

  def sendClientResponse(self, response=""):
    data = response + self.outputTerminator
    self.push(data)
    if self.debugOutputEnabled:
      print >> sys.stderr, "> \"%s\"" % data.encode("string_escape")


class SchottLls:
  def __init__(self):
    self.intensity = 0

  def setIntensity(self, value):
    if value < 0 or value > 255: return
    self.intensity = value

  def getIntensity(self):
    return self.intensity


if __name__ == '__main__':
  try:
    initializeSignalHandlers()
    main(sys.argv)
  except Exception, e:
    if isinstance(e, SystemExit):
      raise e
    else:
      print >> sys.stderr
      print >> sys.stderr, "Error: %s" % e
      sys.exit(1)
